
/**
 * Module dependencies.
 */

var express = require('express');
var routes = require('./routes');
var http = require('http');
var path = require('path');
var sass = require("node-sass");

var app = express();

// all environments
app.set('port', process.env.PORT || 3000);
app.set('views', __dirname + '/views');
app.set('view engine', 'jade');
app.use(sass.middleware({
    src: __dirname + "/public",
    dest: __dirname + "/public",
    outputStyle: "compressed",
    debug: true
}));
app.use(express.favicon());
app.use(express.logger('dev'));
app.use(express.bodyParser({uploadDir: './uploads'}));
app.use(express.methodOverride());
app.use(express.cookieParser('COMP2406 rules!'));
app.use(express.session());
app.use(app.router);
app.use(express.static(path.join(__dirname, 'public')));


// development only
if ('development' == app.get('env')) {
  app.use(express.errorHandler());
}

app.get('/', routes.index);
app.get('/game', routes.game);
app.get('/main', routes.main);
app.get('/editor', routes.editor);
app.get('/roomDetails', routes.roomDetails);
app.post('/deleteRoom', routes.deleteRoom);
app.post('/updateRoom', routes.updateRoom);
app.post('/createRoom', routes.createRoom);
app.post('/importRooms', routes.importRooms);
app.get('/exportRooms', routes.exportRooms);

app.get('/getContents', routes.getContents);
app.post('/start', routes.start);
app.get('/logout', routes.quit);
app.post('/doAction', routes.doAction);

http.createServer(app).listen(app.get('port'), function(){
    console.log('Express server listening on port ' + app.get('port'));
});
