(function() { // local scope

//   ______   ______    ________  _________   ______    ______   ______   ___ __ __
//  /_____/\ /_____/\  /_______/\/________/\ /_____/\  /_____/\ /_____/\ /__//_//_/\
//  \::::_\/_\:::_ \ \ \__.::._\/\__.::.__\/ \:::_ \ \ \:::_ \ \\:::_ \ \\::\| \| \ \
//   \:\/___/\\:\ \ \ \   \::\ \    \::\ \    \:(_) ) )_\:\ \ \ \\:\ \ \ \\:.      \ \
//    \::___\/_\:\ \ \ \  _\::\ \__  \::\ \    \: __ `\ \\:\ \ \ \\:\ \ \ \\:.\-/\  \ \
//     \:\____/\\:\/.:| |/__\::\__/\  \::\ \    \ \ `\ \ \\:\_\ \ \\:\_\ \ \\. \  \  \ \
//      \_____\/ \____/_/\________\/   \__\/     \_\/ \_\/ \_____\/ \_____\/ \__\/ \__\/
//

var roomClick = function() {
    var $roomList = $(".room-list");
    if($roomList.hasClass("exit-picking")) {
        exitRoomPicking(this);
    } else if($roomList.hasClass("multi-delete-picking")) {
        multiDeletePicking(this);
    } else if($roomList.hasClass("key-picking")) {
        keyPicking(this);
    } else if($roomList.hasClass("start-room-picking")) {
        startRoomPicking(this);
    } else {
        defaultRoomAction(this);
    }
}


var defaultRoomAction = function(that) {
    var $this = $(that);
    if($this.hasClass("active")) return;

    var $editRoom = $(".editing-room"),
        $editActions = $(".edit-actions"),
        start = $this.siblings(".active").length === 0,
        timerLength = (start) ? 0 : 400;
    $this.addClass("active");
    $this.siblings().removeClass("active");
    $editRoom.addClass("inactive");
    if(start) $editActions.addClass("inactive");

    setTimeout(function() {
        $.getJSON("/roomDetails?room=" + $this.children("h3").text(), function(room) {
            var fill = ["title", "description"];
            var $text;

            fill.forEach(function(attr) {
                $text = $editRoom.find("[name=" + attr + "]");
                $text.val(room[attr]);
            });

            updateRoomExits(room.exits);
            updateRoomKeys(room.keys);
            $this.attr({"data-room-exits": room.roomExits, "data-room-keys": room.keys});

            $editRoom.removeClass("inactive");
            if(start) $editActions.removeClass("inactive");
        });
    }, timerLength);
}

var updateRoomDescription = function() {
    var $this = $(this),
        $actRoom = $(".room.active");
    $.post("/updateRoom", {room: $actRoom.attr("data-room-title"), description: $this.val()});
    $actRoom.attr("data-room-description", $this.val());
}

var updateName = function() {
    var $this = $(this),
        $actRoom = $(".room.active");
    if($this.val().toLowerCase().replace(/\s+/, "") === $actRoom.attr("data-room-title")) return;
    $.ajax({
        url: "/updateRoom",
        type: "POST",
        data: {room: $actRoom.attr("data-room-title"), newTitle: $this.val()},

        beforeSend: function() {

        },

        success: function(room) {
            console.log($actRoom);
            $actRoom.attr("data-room-title", room.newTitle).find("h3").text($this.val());
        },

        error: function(msg) {
            makeMessage(msg.responseText, "OK, I'll Rename It.", function() {
                setTimeout(function() {
                    $this.focus();
                    document.execCommand("selectAll", false, null);
                }, 100);
            });
        }
    });

}

var cancelEditing = function(e) {
    if(e) e.preventDefault();
    $(".room.active").removeClass("active");
    $(".editing-room").addClass("inactive").children("input, textarea").val("");
    $(".edit-actions").addClass("inactive");
}





//   ______   ______    ________  _________   ______   __     __    ________  _________  ______
//  /_____/\ /_____/\  /_______/\/________/\ /_____/\ /__/\ /__/\  /_______/\/________/\/_____/\
//  \::::_\/_\:::_ \ \ \__.::._\/\__.::.__\/ \::::_\/_\ \::\\:.\ \ \__.::._\/\__.::.__\/\::::_\/_
//   \:\/___/\\:\ \ \ \   \::\ \    \::\ \    \:\/___/\\_\::_\:_\/    \::\ \    \::\ \   \:\/___/\
//    \::___\/_\:\ \ \ \  _\::\ \__  \::\ \    \::___\/_ _\/__\_\_/\  _\::\ \__  \::\ \   \_::._\:\
//     \:\____/\\:\/.:| |/__\::\__/\  \::\ \    \:\____/\\ \ \ \::\ \/__\::\__/\  \::\ \    /____\:\
//      \_____\/ \____/_/\________\/   \__\/     \_____\/ \_\/  \__\/\________\/   \__\/    \_____\/
//

var pickExits = function(e) {
    e.preventDefault();
    var $exits = $(".editing-room [name=exits]").val().split(","),
        $rooms = $(".room");
    $exits.forEach(function(exit) {
        $rooms.filter("[data-room-title=" + exit + "]").addClass("selected");
    });
    $(".room-list").addClass("exit-picking info-box-active");
    $(".exit-picker-actions").addClass("active");
}

var exitRoomPicking = function(that) {
    var $this = $(that);
    if($this.hasClass("active")) return;

    $this.toggleClass("selected");
}

var exitPickerActions = function(e) {
    e.preventDefault();
    var $this = $(this);
    if($(".exit-picking").length === 0) return;
    if($this.hasClass("save-button")) {
        var newExits = [];
        $(".room.selected").each(function() {
            $this = $(this);
            newExits.push($this.attr("data-room-title"));
        });
        updateRoomExits(newExits);
        if(newExits.length === 0) newExits.push("");
        $.post("/updateRoom", {room: $(".room.active").attr("data-room-title"), exits: newExits});
    }
    $(".info-box.active").removeClass("active");
    $(".room.selected").removeClass("selected");
    $(".room-list").removeClass("exit-picking info-box-active");
}

var updateRoomExits = function(exitArray) {
    if(!exitArray) exitArray = [];
    var numExits = exitArray.length,
        $editRoom = $(".editing-room"),
        $exitDeets = $(".exit-details");
    switch(numExits) {
        case 0:
            $exitDeets.html("No exits selected");
            break;
        case 1:
            $exitDeets.html("1 exit selected");
            break;
        default:
            $exitDeets.html(numExits + " exits selected");
    }
    $editRoom.find("[name=exits]").val(exitArray);
    $(".room.active").attr("data-room-exits", exitArray);
}





//   ______   ______    ________  _________   ___   ___   ______   __  __   ______
//  /_____/\ /_____/\  /_______/\/________/\ /___/\/__/\ /_____/\ /_/\/_/\ /_____/\
//  \::::_\/_\:::_ \ \ \__.::._\/\__.::.__\/ \::.\ \\ \ \\::::_\/_\ \ \ \ \\::::_\/_
//   \:\/___/\\:\ \ \ \   \::\ \    \::\ \    \:: \/_) \ \\:\/___/\\:\_\ \ \\:\/___/\
//    \::___\/_\:\ \ \ \  _\::\ \__  \::\ \    \:. __  ( ( \::___\/_\::::_\/ \_::._\:\
//     \:\____/\\:\/.:| |/__\::\__/\  \::\ \    \: \ )  \ \ \:\____/\ \::\ \   /____\:\
//      \_____\/ \____/_/\________\/   \__\/     \__\/\__\/  \_____\/  \__\/   \_____\/
//

var pickKeys = function(e) {
    e.preventDefault();
    if($(".active.start").length > 0) {
        makeMessage("Sorry, you can't lock your start room.", "Rats!");
        return;
    }
    var $keys = $(".editing-room [name=keys]").val().split(","),
        $rooms = $(".room");
    $keys.forEach(function(key) {
        $rooms.filter("[data-room-title=" + key + "]").addClass("keyed");
    });
    $(".room-list").addClass("key-picking info-box-active");
    $(".key-picker-actions").addClass("active");
}

var keyPicking = function(that) {
    var $this = $(that);
    if($this.hasClass("active")) return;

    $this.toggleClass("keyed");
}

var exitKeyPicking = function(e) {
    e.preventDefault();
    var $this = $(this);
    if($this.hasClass("save-button")) {
        var newKeys = [];
        $(".room.keyed").each(function() {
            $this = $(this);
            newKeys.push($this.attr("data-room-title"));
        });
        updateRoomKeys(newKeys);
        if(newKeys.length === 0) newKeys.push("");
        $.post("/updateRoom", {room: $(".room.active").attr("data-room-title"), keys: newKeys});
    }
    $(".info-box.active").removeClass("active");
    $(".room.keyed").removeClass("keyed");
    $(".room-list").removeClass("key-picking info-box-active")
}

var updateRoomKeys = function(keyArray) {
    if(!keyArray) keyArray = [];
    var numKeys = keyArray.length,
        $editRoom = $(".editing-room"),
        $keyDeets = $editRoom.find(".key-details");
    switch(numKeys) {
        case 0:
            $keyDeets.html("Room isn't locked");
            break;
        case 1:
            $keyDeets.html("1 key");
            break;
        default:
            $keyDeets.html(numKeys + " keys");
    }
    $editRoom.find("[name=keys]").val(keyArray);
    $(".room.active").attr("data-room-keys", keyArray);
}






//   ______   ______   __       ______   _________  ______
//  /_____/\ /_____/\ /_/\     /_____/\ /________/\/_____/\
//  \:::_ \ \\::::_\/_\:\ \    \::::_\/_\__.::.__\/\::::_\/_
//   \:\ \ \ \\:\/___/\\:\ \    \:\/___/\  \::\ \   \:\/___/\
//    \:\ \ \ \\::___\/_\:\ \____\::___\/_  \::\ \   \::___\/_
//     \:\/.:| |\:\____/\\:\/___/\\:\____/\  \::\ \   \:\____/\
//      \____/_/ \_____\/ \_____\/ \_____\/   \__\/    \_____\/
//

var deleteRoom = function(roomTitles) {
    roomTitles = ("" + roomTitles).toLowerCase().replace(/\s+/, "");

    $.post("/deleteRoom", {rooms: roomTitles}, function(changes) {

        roomTitles.split(",").forEach(function(theRoom) {
            $(".room[data-room-title=" + theRoom + "]").addClass("deleting").removeClass("active");
        });

        setTimeout(function() {
            $(".deleting").remove();
        }, 400);
    })
}

var singleDelete = function(e) {
    if($(".room.active").hasClass("start")) {
        makeMessage("Sorry, you can't delete your \"Start\" room.", "Fine...");
        return;
    }
    e.preventDefault();
    cancelEditing();
    deleteRoom($(".editing-room input[name=title]").val());
}

var multiDelete = function(e) {
    $(".room.active").addClass("to-delete");
    e.preventDefault();
    cancelEditing();
    $(".room-list").addClass("multi-delete-picking info-box-active");
    $(".multi-delete-actions").addClass("active");
}

var multiDeletePicking = function(that) {
    var $this = $(that);
    $this.toggleClass("to-delete");
}

var exitMultiDelete = function(e) {
    e.preventDefault();
    if($(".start.to-delete").length > 0) {
        makeMessage("Sorry, you can't delete your \"Start\" room.", "Fine...", function() {
            setTimeout(function() {
                $(".start.to-delete").removeClass("to-delete");
            }, 100);
        });
        return;
    }
    var $this = $(this);
    if($this.hasClass("delete-button")) {
        var deleteRooms = [];
        $(".room.to-delete").each(function() {
            deleteRooms.push($(this).attr("data-room-title"));
        });
        deleteRoom(deleteRooms);
    }
    $(".info-box.active").removeClass("active");
    $(".room.to-delete").removeClass("to-delete");
    $(".room-list").removeClass("multi-delete-picking info-box-active")
}







//   ___   __    ______   __ __ __       ______    ______   ______   ___ __ __
//  /__/\ /__/\ /_____/\ /_//_//_/\     /_____/\  /_____/\ /_____/\ /__//_//_/\
//  \::\_\\  \ \\::::_\/_\:\\:\\:\ \    \:::_ \ \ \:::_ \ \\:::_ \ \\::\| \| \ \
//   \:. `-\  \ \\:\/___/\\:\\:\\:\ \    \:(_) ) )_\:\ \ \ \\:\ \ \ \\:.      \ \
//    \:. _    \ \\::___\/_\:\\:\\:\ \    \: __ `\ \\:\ \ \ \\:\ \ \ \\:.\-/\  \ \
//     \. \`-\  \ \\:\____/\\:\\:\\:\ \    \ \ `\ \ \\:\_\ \ \\:\_\ \ \\. \  \  \ \
//      \__\/ \__\/ \_____\/ \_______\/     \_\/ \_\/ \_____\/ \_____\/ \__\/ \__\/
//

var newRoom = function(e) {
    e.preventDefault();
    if($(".exit-picking, .key-picking").length === 0) cancelEditing();
    var $newRoom = $("<div class=\"room new\"><h3 contenteditable=\"true\">New Room</h3></div>").insertBefore($(this));
    $editField = $newRoom.find("[contenteditable=true]").focus();
    document.execCommand("selectAll", false, null);
}

var newRoomSave = function() {
    var $this = $(this);
    $.ajax({
        url: "/createRoom",
        type: "POST",
        data: {"room": $this.text()},

        beforeSend: function() {

        },

        success: function(room) {
            var $roomCard = $this.attr("contenteditable", "false").parent();
            $roomCard.removeClass("new").attr("data-room-title", room.name);
        },

        error: function(msg) {
            makeMessage(msg.responseText, "OK, I'll Rename It.", function() {
                setTimeout(function() {
                    $this.focus();
                    document.execCommand("selectAll", false, null);
                }, 100);
            });
        }
    });
}






//   ______   ______    ________  _________   ______   _________  ________   ______   _________
//  /_____/\ /_____/\  /_______/\/________/\ /_____/\ /________/\/_______/\ /_____/\ /________/\
//  \::::_\/_\:::_ \ \ \__.::._\/\__.::.__\/ \::::_\/_\__.::.__\/\::: _  \ \\:::_ \ \\__.::.__\/
//   \:\/___/\\:\ \ \ \   \::\ \    \::\ \    \:\/___/\  \::\ \   \::(_)  \ \\:(_) ) )_ \::\ \
//    \::___\/_\:\ \ \ \  _\::\ \__  \::\ \    \_::._\:\  \::\ \   \:: __  \ \\: __ `\ \ \::\ \
//     \:\____/\\:\/.:| |/__\::\__/\  \::\ \     /____\:\  \::\ \   \:.\ \  \ \\ \ `\ \ \ \::\ \
//      \_____\/ \____/_/\________\/   \__\/     \_____\/   \__\/    \__\/\__\/ \_\/ \_\/  \__\/
//

var startRoomSelect = function(e) {
    var $selectedRoom = $(".room.active");
    if($selectedRoom.length > 0) {
        if($selectedRoom.attr("data-room-keys") !== "") {
            makeMessage("Sorry, that room is locked and can't be used as a start room.", "Aw, Shucks!");
            return;
        }
        $selectedRoom.addClass("start").siblings(".start").removeClass("start");
        $.post("/updateRoom", {room: $selectedRoom.attr("data-room-title"), start: true});
    } else {
        e.preventDefault();
        cancelEditing();
        $(".room-list").addClass("start-room-picking info-box-active");
        $(".start-picker-actions").addClass("active");
    }
}

var startRoomPicking = function(that) {
    if($(that).attr("data-room-keys") !== "") {
        makeMessage("Sorry, that room is locked and can't be used as a start room.", "Aw, Shucks!");
    } else {
        $(that).addClass("start").siblings(".start").removeClass("start");
        $.post("/updateRoom", {room: $(".room.start").attr("data-room-title"), start: true});
    }
}

var exitStartSelect = function(e) {
    e.preventDefault();
    $(".info-box.active").removeClass("active");
    $(".room-list").removeClass("start-room-picking info-box-active");
}






//   ______   ______    ________   _______      ___   __    ___    ______   ______    ______   ______
//  /_____/\ /_____/\  /_______/\ /______/\    /__/\ /__/\ /__/\  /_____/\ /_____/\  /_____/\ /_____/\
//  \:::_ \ \\:::_ \ \ \::: _  \ \\::::__\/__  \::\_\\  \ \\::\ \ \:::_ \ \\:::_ \ \ \:::_ \ \\:::_ \ \
//   \:\ \ \ \\:(_) ) )_\::(_)  \ \\:\ /____/\  \:. `-\  \ \\:_\/  \:\ \ \ \\:(_) ) )_\:\ \ \ \\:(_) \ \
//    \:\ \ \ \\: __ `\ \\:: __  \ \\:\\_  _\/   \:. _    \ \       \:\ \ \ \\: __ `\ \\:\ \ \ \\: ___\/
//     \:\/.:| |\ \ `\ \ \\:.\ \  \ \\:\_\ \ \    \. \`-\  \ \       \:\/.:| |\ \ `\ \ \\:\_\ \ \\ \ \
//      \____/_/ \_\/ \_\/ \__\/\__\/ \_____\/     \__\/ \__\/        \____/_/ \_\/ \_\/ \_____\/ \_\/
//

var dragOn = function(e) {
    e.stopPropagation();
    e.preventDefault();
    $(this).addClass("dragged");
}

var dragOff = function(e) {
    e.stopPropagation();
    e.preventDefault();
    $(this).removeClass("dragged");
}

var dragOver = function(e) {
    e.stopPropagation();
    e.preventDefault();
}

var dropFile = function(e) {
    e.preventDefault();
    e.stopPropagation();

    var $this = $(this),
        f = e.originalEvent.dataTransfer.files;

    $this.removeClass("dragged");

    var files = new FormData();
    files.append("file", f[0]);

    $.ajax({
        url: "/importRooms",
        type: "POST",
        data: files,
        cache: false,
        contentType: false,
        processData: false,

        beforeSend: function() {

        },

        success: function(newRooms) {
            $(".room:not(.new-room)").remove();
            for(r in newRooms) {
                if(newRooms.hasOwnProperty(r)) {
                    addRoom(newRooms[r], r);
                }
            }
            makeMessage("Rooms were imported successfully!", "Hurray!", function() {
                setTimeout(function() {
                    dismissDropZone();
                }, 150);
            });
        },

        error: function(msg) {
            makeMessage(msg.responseText)
        }
    });
}

var addRoom = function(room, name) {
    var r = "<div class=\"room\" data-room-title=\"" + name +"\"" +
            "data-room-description=\"" + room.description +"\"" +
            "data-room-keys=\"" + room.keys +"\"" +
            "data-room-exits=\"" + room.roomExits +"\"" +
            "><h3>" + room.title + "</h3></div>";
    r = $(r).insertBefore(".new-room");
    if(room.start) {
        r.addClass("start");
    }
}

var activateDropZone = function(e) {
    e.preventDefault();
    cancelEditing();
    $(".import-drag-drop").addClass("active");
    $(".room-list").addClass("import-picking info-box-active");
    $(".import-actions").addClass("active");
}

var dismissDropZone = function(e) {
    if(e) e.preventDefault();
    var $dragger = $(".import-drag-drop");
    $dragger.css("opacity", 0);
    $(".info-box.active").removeClass("active");
    setTimeout(function() {
        $dragger.removeClass("active").css("opacity", "");
        $(".room-list").removeClass("import-picking info-box-active");
    }, 200);
}





//   ______   __     __   ______   ______   ______   _________
//  /_____/\ /__/\ /__/\ /_____/\ /_____/\ /_____/\ /________/\
//  \::::_\/_\ \::\\:.\ \\:::_ \ \\:::_ \ \\:::_ \ \\__.::.__\/
//   \:\/___/\\_\::_\:_\/ \:(_) \ \\:\ \ \ \\:(_) ) )_ \::\ \
//    \::___\/_ _\/__\_\_/\\: ___\/ \:\ \ \ \\: __ `\ \ \::\ \
//     \:\____/\\ \ \ \::\ \\ \ \    \:\_\ \ \\ \ `\ \ \ \::\ \
//      \_____\/ \_\/  \__\/ \_\/     \_____\/ \_\/ \_\/  \__\/
//

var exportRooms = function(e) {
    e.preventDefault();
    window.open("/exportRooms");
}






//   ___ __ __   ______   ______   ______   ________   _______    ______
//  /__//_//_/\ /_____/\ /_____/\ /_____/\ /_______/\ /______/\  /_____/\
//  \::\| \| \ \\::::_\/_\::::_\/_\::::_\/_\::: _  \ \\::::__\/__\::::_\/_
//   \:.      \ \\:\/___/\\:\/___/\\:\/___/\\::(_)  \ \\:\ /____/\\:\/___/\
//    \:.\-/\  \ \\::___\/_\_::._\:\\_::._\:\\:: __  \ \\:\\_  _\/ \::___\/_
//     \. \  \  \ \\:\____/\ /____\:\ /____\:\\:.\ \  \ \\:\_\ \ \  \:\____/\
//      \__\/ \__\/ \_____\/ \_____\/ \_____\/ \__\/\__\/ \_____\/   \_____\/
//

var makeMessage = function(msg, dismissText, callback) {
    dismissText = dismissText || "Dismiss";
    var $errMsg = $(".error-overlay").css("display", "block").children();
    $errMsg.find(".dismiss-button").text(dismissText);
    $errMsg.find("p:first").text(msg);
    setTimeout(function() {
        $(".error-overlay").addClass("error-active");
    }, 1);

    if(callback) {
        $(".dismiss-button").on("click", callback);
    }
}

var dismissMessage = function(e) {
    e.preventDefault();
    $(".error-overlay").removeClass("error-active");
    setTimeout(function() {
        $(".error-overlay").css("display", "none");
        $(".dimiss-button").off("click");
        $(".dimiss-button").on("click", dismissMessage);
    }, 250);
}






//    _______    ________  ___   __    ______    ________  ___   __    _______
//  /_______/\  /_______/\/__/\ /__/\ /_____/\  /_______/\/__/\ /__/\ /______/\
//  \::: _  \ \ \__.::._\/\::\_\\  \ \\:::_ \ \ \__.::._\/\::\_\\  \ \\::::__\/__
//   \::(_)  \/_   \::\ \  \:. `-\  \ \\:\ \ \ \   \::\ \  \:. `-\  \ \\:\ /____/\
//    \::  _  \ \  _\::\ \__\:. _    \ \\:\ \ \ \  _\::\ \__\:. _    \ \\:\\_  _\/
//     \::(_)  \ \/__\::\__/\\. \`-\  \ \\:\/.:| |/__\::\__/\\. \`-\  \ \\:\_\ \ \
//      \_______\/\________\/ \__\/ \__\/ \____/_/\________\/ \__\/ \__\/ \_____\/
//

$(document).ready(function() {

    $(document).on("blur", "[name=description]", updateRoomDescription);
    $(document).on("blur", "[name=title]", updateName);
    $(document).on("click", ".done-edits", cancelEditing);

    $(document).on("click", ".room:not(.new-room)", roomClick);

    $(document).on("click", ".delete-room", singleDelete);
    $(document).on("click", ".multi-delete", multiDelete);
    $(document).on("click", ".multi-delete-actions a", exitMultiDelete);

    $(document).on("click", ".new-room", newRoom);
    $(document).on("blur", ".room [contenteditable=true]", newRoomSave);

    $(document).on("click", ".room-exit-mode-toggle", pickExits);
    $(document).on("click", ".exit-picker-actions a", exitPickerActions);

    $(document).on("click", ".key-picker-actions a", exitKeyPicking);
    $(document).on("click", ".room-key-mode-toggle", pickKeys);

    $(document).on("click", ".set-start-room", startRoomSelect);
    $(document).on("click", ".start-picker-actions a", exitStartSelect);

    $(document).on("click", ".import-rooms", activateDropZone);
    $(document).on("click", ".cancel-import", dismissDropZone);
    $(document).on("dragenter", ".import-drag-drop", dragOn);
    $(document).on("dragleave", ".import-drag-drop", dragOff);
    $(document).on("dragover", ".import-drag-drop", dragOver);
    $(document).on("drop", ".import-drag-drop", dropFile);

    $(document).on("click", ".export-rooms", exportRooms);

    $(document).on("click", ".dismiss-button", dismissMessage);

});

})();