var sanitize = require('validator').sanitize;
var rooms = {}; //require('../rooms.js').rooms;
var fs = require("fs");


var mongoose = require('mongoose');
var db = mongoose.connection;
var Room;

mongoose.connect('mongodb://localhost/db-rooms-game');
db.on('error', console.error.bind(console, 'MongoDB connection error:'));
db.once('open', function() {
    var roomSchema = mongoose.Schema({
        r:              String, // Title, lowercase and whitespace stripped (URLs)
        title:          String, // Full name (capitalization and spaces)
        description:    String,
        exits:          [String],
        keys:           [String],
        start:          Boolean
    });

    Room = mongoose.model("Room", roomSchema);

    // var bootstrap = new Room({
    //     r:              "bridge",
    //     title:          "Bridge",
    //     description:    "You are on the Bridge.  There are big comfy chairs and a big screen here.",
    //     roomExits:      [],
    //     keys:           [],
    //     start:          true
    // });

    // bootstrap.save();
});


//   ______   ________   _______    ______   ______
//  /_____/\ /_______/\ /______/\  /_____/\ /_____/\
//  \:::_ \ \\::: _  \ \\::::__\/__\::::_\/_\::::_\/_
//   \:(_) \ \\::(_)  \ \\:\ /____/\\:\/___/\\:\/___/\
//    \: ___\/ \:: __  \ \\:\\_  _\/ \::___\/_\_::._\:\
//     \ \ \    \:.\ \  \ \\:\_\ \ \  \:\____/\ /____\:\
//      \_\/     \__\/\__\/ \_____\/   \_____\/ \_____\/
//

function index(req, res) {
    if (req.session.player) {
        res.redirect("/main");
    } else {
	   res.render('index', { title: 'COMP 2406 Adventure Demo', error: req.query.error});
    }
}

function start(req, res) {
    var player = req.body.player;
    if(!player || player.length === 0) {
        res.redirect("/?error=Player name can't be blank.")
    } else {
        // Sanitize the player name as it may be displayed
        req.session.player = sanitize(player).escape();
        res.redirect("/main")
    }
}

function main(req, res) {

    res.render("main.jade", {username: req.session.player});
}

function logout(req, res) {
    req.session.destroy(function(err) {
        if(err) {
            console.log("Error: %s", err);
        }
    });
    res.redirect("/");
}

function game(req, res) {
    if(req.session.player) {
        if(!req.session.currentRoom) {

            Room.findOne({start: true}, function(err, room) {
                if(!room) {
                    res.redirect("/editor?error=norooms");
                    return;
                }
                req.session.currentRoom = room.r;
                Room.find({keys: room.r}, function(err, keyRooms) {

                    if(keyRooms) {
                        var keys = [];
                        keyRooms.forEach(function(keyRoom) {
                            keys.push(keyRoom.r);
                        });

                        req.session.keys = "" + keys;
                        res.render("room.jade", {title: "Adventure Demo"});
                    }
                });
            });
        } else {
            res.render("room.jade", {title: "Adventure Demo"});
        }
    } else {
        res.redirect("/");
    }
}

function editor(req, res) {
    if(req.session.player) {
        req.session.currentRoom = undefined;
        req.session.keys = undefined;
        Room.find(function(err, allRooms) {
            if(req.query.error) {
                res.render("editor.jade", {title: "Adventure Editor", curRooms: allRooms, error: "no rooms"});
            } else {
                res.render("editor.jade", {title: "Adventure Editor", curRooms: allRooms});
            }
        })
    } else {
        res.redirect("/");
    }
}



//   ______   ______    ________  _________   ___   ___   ______   __       ______
//  /_____/\ /_____/\  /_______/\/________/\ /__/\ /__/\ /_____/\ /_/\     /_____/\
//  \::::_\/_\:::_ \ \ \__.::._\/\__.::.__\/ \::\ \\  \ \\::::_\/_\:\ \    \:::_ \ \
//   \:\/___/\\:\ \ \ \   \::\ \    \::\ \    \::\/_\ .\ \\:\/___/\\:\ \    \:(_) \ \
//    \::___\/_\:\ \ \ \  _\::\ \__  \::\ \    \:: ___::\ \\::___\/_\:\ \____\: ___\/
//     \:\____/\\:\/.:| |/__\::\__/\  \::\ \    \: \ \\::\ \\:\____/\\:\/___/\\ \ \
//      \_____\/ \____/_/\________\/   \__\/     \__\/ \::\/ \_____\/ \_____\/ \_\/
//

// Gets the details (title, description, exits and keys)
// of the querried room
function roomDetails(req, res) {
    var query = (req.query.room).toLowerCase().replace(/\s+/, "");
    Room.findOne({r: query}, function(err, room) {
        res.send(room);
    });
}

// Updates one or more of the room's details
function updateRoom(req, res) {
    var roomTitle = (req.body.room).toLowerCase().replace(/\s+/, "");

    var update = {};

    if(req.body.exits) {
        if(req.body.exits[0] !== "") {
            update.exits = req.body.exits;
        }
    }

    if(req.body.keys) {
        if(req.body.keys[0] !== "") {
            update.keys = req.body.keys;
        }
    }

    if(req.body.description || req.body.description === "") {
        update.description = req.body.description;
    }

    if(req.body.start) {
        update.start = true;
    }

    var newRoomTitle = (req.body.newTitle) ? (req.body.newTitle).toLowerCase().replace(/\s+/, "") : roomTitle;

    Room.findOne({r: newRoomTitle}, function(err, room) {
        if(newRoomTitle !== roomTitle && room) {
            res.status(400).send("A room by the name of \"" + req.body.newTitle + "\" already exists.");
            return;
        } else {

            Room.update({r: roomTitle}, { $set: update }, function(err) {
                if(update.start) {
                    Room.find({start: true, r: {$ne: roomTitle}}, function(err, oldStarts) {
                        oldStarts.forEach(function(oldStart) {
                            oldStart.start = false;
                            oldStart.save();
                        });
                    });
                }

                if(newRoomTitle !== "") {
                    Room.find({exits: roomTitle}, function(err, docs) {
                        docs.forEach(function(doc) {
                            doc.exits.splice(doc.exits.indexOf(roomTitle), 1);
                            doc.exits.push(newRoomTitle);
                            doc.save();
                        });

                        Room.find({keys: roomTitle}, function(err, docs) {
                            docs.forEach(function(doc) {
                                doc.keys.splice(doc.keys.indexOf(roomTitle), 1);
                                doc.keys.push(newRoomTitle);
                                doc.save();
                            });
                            res.send(room);
                            return;
                        });
                    });
                } else {
                    res.send(room);
                }
            });
        }
    });
}

// Deletes a room from the database, correcting all
// related room's containsKeys/ roomExits arrays
function deleteRoom(req, res) {
    var rooms = (req.body.rooms).split(",");

    Room.remove({r: {$in: rooms}}, function(err) {

        Room.find({$or: [{keys: {$in: rooms}}, {exits: {$in: rooms}}]}, function(err, theRooms) {
            var position;
            console.log(theRooms);

            theRooms.forEach(function(theRoom) {
                if(rooms.indexOf(theRoom.r) < 0) {
                    rooms.forEach(function(removeRoom) {
                        position = theRoom.exits.indexOf(removeRoom);
                        if(position >= 0) theRoom.exits.splice(position, 1);

                        position = theRoom.keys.indexOf(removeRoom);
                        if(position >= 0) theRoom.keys.splice(position, 1);
                    });

                    theRoom.save();
                }
            });
            res.send({status: "success"});
        });
    }).exec();
}

// Imports the rooms from a JSON, checking for a valid file
// and performing some basic checks to ensure that the rooms
// don't have major issues
function importRooms(req, res) {
    var path = req.files.file.path;
    fs.readFile(path, "utf-8", function(err, content) {
        if(err) throw err;
        try {
            var newRooms = JSON.parse(content),
                cleanedRooms = {},
                targetRoomName,
                startRoomFound = false,
                position,
                containsRoom;


            for(var r in newRooms) {

                room = newRooms[r];
                if(!room) continue;

                // Return if there is no title
                if(!room.title) {
                    res.status(400).send("Room titles can't be blank.");
                    return;
                }

                targetRoomName = room.title.toLowerCase().replace(/\s+/, "");
                room.r = targetRoomName;

                // Clean the room's keys array
                if(!room.keys) {
                    room.keys = [];
                } else if(room.keys.length > 0) {
                    room.keys.forEach(function(keyRoom) {
                        // If the key doesn't exist anywhere else, remove it
                        if(!(cleanedRooms[keyRoom] || newRooms[keyRoom])) {
                            room.keys.splice(i, 1);
                        }
                    });
                }

                // Clean the room's exits
                if(!room.exits) {
                    room.exits = [];
                } else if(room.exits.length > 0) {
                    room.exits.forEach(function(exit, i) {
                        // If the room doesn't exist anywhere else, remove it
                        if(!(cleanedRooms[exit] || newRooms[exit])) {
                            room.exits.splice(i, 1);
                        }
                    });
                }

                // Clean the room's description
                if(!room.description) room.description = "";

                // Check to ensure a start room is not locked
                if(!room.start) {
                    room.start = false;
                } else if(room.keys.length > 0) {
                    res.status(400).send("\"" + room.title + "\" is the start room and, therefore, can't be locked.");
                    return;
                } else {
                    if(startRoomFound) {
                        room.start = false;
                    } else {
                        startRoomFound = true;
                    }
                }

                cleanedRooms[targetRoomName] = room;
            }

            // If no start room was found, set it to start, or to the first non-locked room we iterate over
            if(!startRoomFound) {
                if(cleanedRooms["start"]) {
                    cleanedRooms["start"].start = true;
                } else {
                    for(var r in cleanedRooms) {
                        if(cleanedRooms[r].keys.length < 1) {
                            cleanedRooms[r].start = true;
                            startRoomFound = true;
                            break;
                        }
                    }
                    if(!startRoomFound) {
                        res.status(400).send("All of your rooms are locked; you must leave at least one unlocked.");
                        return;
                    }
                }
            }

            // Reassign the room
            var aRoom;
            Room.remove({}, function(err) {
                res.status(400).send("Error deleting the entire database.");

                for(var r in cleanedRooms) {
                    aRoom = new Room(cleanedRooms[r]);
                    aRoom.save(function(err, theRoom) {
                        if(err) {
                            console.log("Failure importing " + theRoom.title);
                        }
                    });
                }
            });

            // Return the new rooms for updating the editor interface
            res.send(cleanedRooms);

            // Delete the temp file
            fs.unlink(path);

        } catch(e) {
            console.log(e);
            res.status(400).send("The suplied file is not valid JSON.");
        }
    });
}

// Exports the rooms to a text file in JSON format
function exportRooms(req, res) {
    var path = "exports/rooms.txt";
    Room.find(function(err, rooms) {
        var allRooms = {};
        rooms.forEach(function(room) {
            allRooms[room.r] = room;
        });

        fs.writeFile(path, JSON.stringify(allRooms, null, '\t'), function(err) {
            if(!err) {
                res.download(path, function() {
                    fs.unlink(path);
                });
            }
        });
    });
}

// Create a new room, checking for an exiting room of that name
function createRoom(req, res) {
    var room = (req.body.room).toLowerCase().replace(/\s+/, "");

    Room.findOne({r: room}, function(err, theRoom) {
        if(theRoom) {
            res.status(400).send("A room named \"" + req.body.room + "\" already exists.");
            return;
        } else {
            var aNewRoom = {
                r:              room,
                title:          req.body.room,
                description:    req.body.description || "",
                exits:          req.body.exits || [],
                keys:           req.body.keys || [],
                start:          false
            }
            var newRoom = new Room(aNewRoom);
            newRoom.save(function(err) {
                res.send(aNewRoom);
            });
        }
    });
}



//   ______    ______   ______   ___ __ __       ___   ___   ______   __       ______
//  /_____/\  /_____/\ /_____/\ /__//_//_/\     /__/\ /__/\ /_____/\ /_/\     /_____/\
//  \:::_ \ \ \:::_ \ \\:::_ \ \\::\| \| \ \    \::\ \\  \ \\::::_\/_\:\ \    \:::_ \ \
//   \:(_) ) )_\:\ \ \ \\:\ \ \ \\:.      \ \    \::\/_\ .\ \\:\/___/\\:\ \    \:(_) \ \
//    \: __ `\ \\:\ \ \ \\:\ \ \ \\:.\-/\  \ \    \:: ___::\ \\::___\/_\:\ \____\: ___\/
//     \ \ `\ \ \\:\_\ \ \\:\_\ \ \\. \  \  \ \    \: \ \\::\ \\:\____/\\:\/___/\\ \ \
//      \_\/ \_\/ \_____\/ \_____\/ \__\/ \__\/     \__\/ \::\/ \_____\/ \_____\/ \_\/
//

// Manages attempts to move to a new room, throwing an error
// to the client when a room is locked
function doAction(req, res) {
    var action = req.body.action;
    var room = req.body.room;

    // If the action is to move and the room exists
    if (action === "move") {

        Room.findOne({r: room}, function(err, theRoom) {
            if(theRoom) {
                // If the room is locked, return
                if(theRoom.keys.length > 0 && !(req.session.keys && req.session.keys.match(room))) {
                    res.send(theRoom.title + " is locked. Find a key for it and come back later!");
                    return;
                } else {
                    // Else, move the current room name
                    req.session.currentRoom = room;

                    // Update the keys that the user has available and assign to req.session.keys
                    Room.find({keys: room}, function(err, keyContainsRooms) {
                        var newKeys = [],
                            currentKeys = (req.session.keys) ? (req.session.keys).split(",") : [],
                            returnKeys = [],
                            position;

                        keyContainsRooms.forEach(function(aRoom) {
                            newKeys.push(aRoom.r);
                        });

                        if(newKeys) {
                            currentKeys.forEach(function(key) {
                                returnKeys.push(key);
                            });

                            newKeys.forEach(function(key) {
                                position = returnKeys.indexOf(key);
                                if(position < 0) {
                                    returnKeys.push(key);
                                }
                            });

                            req.session.keys = "" + returnKeys;
                        }

                        res.send("Success");
                    });
                }
            } else {
                res.send("Error: InvalidRoom");
            }
        });
    } else {
        res.send("Error: InvalidAction");
    }
}

// Returns all details required to update a room page: the room's
// details, which of its exits are locked, the prettified names of
// its exits, and the user's collected keys
function getContents(req, res) {
    // Get everything required to update the room UI:
    // - Current keys
    // - Details of current room
    // - Which exits are locked
    // - Names and titles of the current room
    var keyArr = (req.session.keys) ? (req.session.keys).split(",") : [];

    Room.findOne({r: req.session.currentRoom}, function(err, room) {

        room.description = room.description.replace(/\<\?player\>/i, req.session.player);

        var curRoomExits = room.exits,
            curRoomExitNames = {},
            keyTitles = [],
            lockedExits = {};

        Room.find({r: {$in: curRoomExits}}, function(err, lockedRooms) {
            lockedRooms.forEach(function(theRoom) {
                lockedExits[theRoom.r] = theRoom.keys.length > 0;
                curRoomExitNames[theRoom.r] = theRoom.title;
            });


            Room.find({r: {$in: keyArr}}, function(err, keyRooms) {
                keyRooms.forEach(function(theRoom) {
                    keyTitles.push(theRoom.title);
                });

                res.send({
                    "room": room,
                    "exits": curRoomExitNames,
                    "keys": keyArr,
                    "keyTitles": keyTitles,
                    "lockedExits": lockedExits
                });
            });
        });
    });
}




//    _______    ________  ___   __    ______    ________  ___   __    _______
//  /_______/\  /_______/\/__/\ /__/\ /_____/\  /_______/\/__/\ /__/\ /______/\
//  \::: _  \ \ \__.::._\/\::\_\\  \ \\:::_ \ \ \__.::._\/\::\_\\  \ \\::::__\/__
//   \::(_)  \/_   \::\ \  \:. `-\  \ \\:\ \ \ \   \::\ \  \:. `-\  \ \\:\ /____/\
//    \::  _  \ \  _\::\ \__\:. _    \ \\:\ \ \ \  _\::\ \__\:. _    \ \\:\\_  _\/
//     \::(_)  \ \/__\::\__/\\. \`-\  \ \\:\/.:| |/__\::\__/\\. \`-\  \ \\:\_\ \ \
//      \_______\/\________\/ \__\/ \__\/ \____/_/\________\/ \__\/ \__\/ \_____\/
//

exports.index = index;
exports.start = start;
exports.main = main;
exports.quit = logout;
exports.game = game;
exports.editor = editor;
exports.doAction = doAction;
exports.getContents = getContents;
exports.roomDetails = roomDetails;
exports.deleteRoom = deleteRoom;
exports.updateRoom = updateRoom;
exports.createRoom = createRoom;
exports.importRooms = importRooms;
exports.exportRooms = exportRooms;